use log::info;
use spoticord_api_grpc::spoticord_api::service::{PlayModes, PlayPlaylistRequest};
use spoticord_player::playback_control::{PlayPlaylist, StartAt};
use spoticord_session::manager::{SessionManager, SessionQuery};
use tonic::{Request, Response, Status};

pub struct SpoticordApi {
    pub session: SessionManager,
}

#[tonic::async_trait]
impl spoticord_api_grpc::spoticord_api::service::spoticord_api_server::SpoticordApi
    for SpoticordApi
{
    async fn play_playlist(
        &self,
        request: Request<PlayPlaylistRequest>,
    ) -> Result<Response<spoticord_api_grpc::spoticord_api::service::Response>, Status> {
        let playlist = request.into_inner();
        info!(
            "Playing {} for {}",
            playlist.playlist_uri, playlist.discord_user_id
        );

        let session = self
            .session
            .get_session(SessionQuery::Owner(playlist.discord_user_id.into()))
            .unwrap();

        let shuffle = match playlist.order() {
            PlayModes::Shuffle => true,
            PlayModes::InOrder => false,
        };

        let play_playlist = PlayPlaylist {
            uri: playlist.playlist_uri,
            shuffle,
            repeat: true,
            start_at: StartAt::FirstSong,
        };

        session.queue_playlist(play_playlist).await.unwrap();

        let response = spoticord_api_grpc::spoticord_api::service::Response {
            resp: Option::from(
                spoticord_api_grpc::spoticord_api::service::response::Resp::Success(true),
            ),
        };

        Ok(Response::new(response))
    }
}
